// SPDX-License-Identifier: MIT

import { Page, expect } from '@playwright/test';

// helper to frame screenshot with a margin around locator
// Might replace it with something better depending on playwright
// changes: https://github.com/microsoft/playwright/issues/28394
async function get_target(
	page: Page,
	locator?: string,
	margin?: number
): [Page | Locator, null | { x: number, y: number, width: number, height: number }] {
	if (!margin) {
		return [ locator ? page.locator(locator) : page, null ]
	}
	if (!locator) {
		throw "Must have a locator if margin was set";
	}
	var box = await page.locator(locator).boundingBox();
	return [ page, {
		x: box.x - margin,
		y: box.y - margin,
		width: box.width + 2 * margin,
		height: box.height + 2 * margin,
	}];
}

export async function screenshot(
	page: Page,
	opts: {
		// path to store screenshot
		path: string,
		// page locator for partial screenshot
		locator?: string,
		// extra styles if required // playwight 1.42+
		style?: string,
		// disable background? keep by default
		omitBackground?: boolean,
		// mask items e.g. [page.locator('foo')]
		mask?: Array<Locator>,
		// margin around element if set
		margin?: number
	},
) {
	var style = opts.style || "";
	var omitBackground = opts.omitBackground || false;
	if (omitBackground) {
		// need plain background to make background actually transparent
		style += '\nbody { background: none; }';
		// firefox doesn't support omitBackground screenshot property as of playwright 1.41...
		if (page.context().browser().browserType().name() == 'firefox') {
		       	omitBackground = false;
		}
	}

	// get screenshot box area
	var [target, clip] = await get_target(page, opts.locator, opts.margin);


	return await target.screenshot({
		path: opts.path,
		clip: clip,
		omitBackground: omitBackground,
		style: style,
		mask: opts.mask,
		// override default pink
		maskColor: "#222222",
	});
}
